import pathlib
import tempfile

import numpy as np
import pytest


@pytest.fixture(scope='module', params=((3, 2), (4, 3, 2), (5, 4, 3, 2)))
def dimension(request):
    yield request.param


@pytest.fixture(scope='module', params=(np.int8, np.uint8, np.int16, np.uint16, np.int32, np.uint32, np.int64, np.uint64, np.float32, np.float64))
def dtype(request):
    yield request.param


@pytest.fixture(scope='function')
def filepath(suffix):
    with tempfile.NamedTemporaryFile(suffix=suffix, delete=False) as f:
        filepath = pathlib.Path(f.name)
    yield filepath
    for s in (suffix, '.raw', '.zraw'):
        fws = filepath.with_suffix(s)
        if fws.is_file():
            fws.unlink()


@pytest.fixture(scope='function')
def filepath_mha():
    with tempfile.NamedTemporaryFile(suffix='.mha', delete=False) as f:
        filepath = pathlib.Path(f.name)
    yield filepath
    if filepath.is_file():
        filepath.unlink()


@pytest.fixture(scope='module', params=('.mha', '.mhd'))
def suffix(request):
    yield request.param



import metaimageio
import numpy as np
import pytest

pytest.importorskip('imageio')


def test_imageio_v2(filepath):
    try:
        import imageio.v2 as iio
    except ModuleNotFoundError:
        import imageio as iio
    metaimageio.imageio()
    a = (100 * np.random.random_sample((4, 3, 2)))
    iio.imwrite(filepath, a, format='MetaImageIO')
    b = iio.imread(filepath, format='MetaImageIO')
    np.testing.assert_almost_equal(b, a)
    meta = b.meta
    np.testing.assert_almost_equal(meta['DimSize'], [2, 3, 4])


def test_imageio_v3(filepath):
    try:
        import imageio.v3 as iio
    except ModuleNotFoundError:
        pytest.skip()
    from metaimageio.imageio import MetaImageIOPlugin
    a = (100 * np.random.random_sample((4, 3, 2)))
    iio.imwrite(filepath, a, plugin=MetaImageIOPlugin)
    b = iio.imread(filepath, plugin=MetaImageIOPlugin)
    np.testing.assert_almost_equal(b, a)
    meta = iio.immeta(filepath, plugin=MetaImageIOPlugin)
    np.testing.assert_almost_equal(meta['DimSize'], [2, 3, 4])



import metaimageio
import numpy as np


def test_channels(filepath_mha, dimension):
    a = (100 * np.random.random_sample(dimension)).astype(np.uint8)
    metaimageio.write(filepath_mha, a, ElementNumberOfChannels=a.shape[-1])
    b, _ = metaimageio.read(filepath_mha)
    np.testing.assert_almost_equal(b, a)


def test_compression(filepath, dtype):
    a = (100 * np.random.random_sample((4, 3, 2))).astype(dtype)
    metaimageio.write(filepath, a, CompressedData=True)
    b, _ = metaimageio.read(filepath)
    np.testing.assert_almost_equal(b, a)


def test_io(filepath, dtype, dimension):
    a = (100 * np.random.random_sample(dimension)).astype(dtype)
    metaimageio.write(filepath, a)
    b, _ = metaimageio.read(filepath)
    np.testing.assert_almost_equal(b, a)


def test_memmap(filepath_mha, dtype):
    a = (100 * np.random.random_sample((4, 3, 2))).astype(dtype)
    metaimageio.write(filepath_mha, a)
    b, _ = metaimageio.read(filepath_mha, memmap=True)
    np.testing.assert_almost_equal(b, a)
    del b



import numpy as np

from . import reader, writer

EXTENSIONS = '.mha', '.mhd'


try:
    from imageio.core.v3_plugin_api import ImageProperties, PluginV3

    class MetaImageIOPlugin(PluginV3):

        def __init__(self, request, **kwargs):
            super().__init__(request, **kwargs)
            self._filepath = request.get_local_filename()

        def read(self, index=None, **kwargs):
            if index is not None:
                kwargs.setdefault('slices', [index])
            image, _ = reader.read(self._filepath, **kwargs)
            return image

        def write(self, ndimage, **kwargs):
            writer.write(self._filepath, image=ndimage, **kwargs)

        def metadata(self, index=None, exclude_applied=True):
            _ = index, exclude_applied
            _, meta = reader.read(self._filepath, slices=())
            return meta

        def properties(self, index=None):
            _ = index
            meta = self.metadata()
            return ImageProperties(
                shape=tuple(meta['DimSize']),
                dtype=meta['ElementType'],
                n_images=None,
                is_batch=False,
                spacing=tuple(meta['ElementSpacing']))

except ModuleNotFoundError:
    pass


try:
    from imageio import core, formats

    class MetaImageIOFormat(core.Format):

        def _can_read(self, request):
            return request.mode[1] in self.modes and request.extension in EXTENSIONS

        def _can_write(self, request):
            return request.mode[1] in self.modes and request.extension in EXTENSIONS

        class Reader(core.Format.Reader):

            def _open(self, **kwargs):
                _ = kwargs
                self._filepath = self.request.get_local_filename()

            def _close(self):
                pass

            def _get_length(self):
                return np.inf

            def _get_data(self, index, **kwargs):
                _ = kwargs
                if index != 0:
                    raise NotImplementedError('MetaImageIO does not support non-zero indices')
                image, meta = reader.read(self._filepath, **self.request.kwargs)
                if image is None:
                    image = np.array(())
                return image, meta

            def _get_meta_data(self, index):
                if index != 0:
                    raise NotImplementedError('MetaImageIO does not support non-zero indices')
                _, meta = reader.read(self._filepath, slices=())
                return meta

        class Writer(core.Format.Writer):

            def _open(self, **kwargs):
                _ = kwargs
                self._filepath = self.request.get_local_filename()

            def _close(self):
                pass

            def _append_data(self, im, meta):
                meta.pop('ElementDataFile', None)
                meta.update(self.request.kwargs)
                writer.write(self._filepath, image=im, **meta)

            def set_meta_data(self, meta):
                _ = meta
                raise NotImplementedError('MetaImageIO does not support writing meta data')

    def add_format(name='MetaImageIO'):
        names = formats.get_format_names()
        if name.upper() not in names:
            formats.add_format(MetaImageIOFormat(
                name,
                'MetaImageIO',
                ' '.join(EXTENSIONS),
                'iv'))
            formats.sort(name, *names)
        return name

except ModuleNotFoundError:
    pass



import metaimageio
import numpy as np
import pytest


@pytest.fixture(scope='module')
def matlab():
    try:
        import matlab.engine
        engine = matlab.engine.start_matlab()
    except ModuleNotFoundError:
        engine = None
    yield engine
    if engine is not None:
        engine.quit()


def mat2py(x):
    return np.transpose(x, list(reversed(range(np.ndim(x)))))


def py2mat(x):
    return mat2py(x)


def test_channels(matlab, filepath_mha, dimension):
    if matlab is None:
        pytest.skip()
    a = (100 * np.random.random_sample(dimension)).astype(np.uint8)
    matlab.metaimageio.write(str(filepath_mha), py2mat(a), {'ElementNumberOfChannels': a.shape[-1]})
    b, _ = matlab.metaimageio.read(str(filepath_mha), nargout=2)
    b = mat2py(b)
    np.testing.assert_almost_equal(b, a)


def test_io(matlab, filepath_mha, dtype, dimension):
    if matlab is None or dtype in (np.int64, np.uint64):
        pytest.skip()
    a = (100 * np.random.random_sample(dimension)).astype(dtype)
    matlab.metaimageio.write(str(filepath_mha), py2mat(a))
    b, _ = matlab.metaimageio.read(str(filepath_mha), nargout=2)
    b = mat2py(b)
    np.testing.assert_almost_equal(b, a)


def test_io_from_python(matlab, filepath_mha):
    if matlab is None:
        pytest.skip()
    a = (100 * np.random.random_sample((4, 3, 2))).astype(np.uint8)
    metaimageio.write(filepath_mha, a)
    b, _ = matlab.metaimageio.read(str(filepath_mha), nargout=2)
    b = mat2py(b)
    np.testing.assert_almost_equal(b, a)


def test_io_to_python(matlab, filepath_mha):
    if matlab is None:
        pytest.skip()
    a = (100 * np.random.random_sample((4, 3, 2))).astype(np.uint8)
    matlab.metaimageio.write(str(filepath_mha), py2mat(a))
    b, _ = metaimageio.read(filepath_mha)
    np.testing.assert_almost_equal(b, a)


def test_wrapper(matlab, filepath_mha, dtype, dimension):
    if matlab is None or dtype in (np.int32, np.uint32, np.int64, np.uint64, np.float64):
        pytest.skip()
    a = (100 * np.random.random_sample(dimension)).astype(dtype)
    matlab.metaimageio.py.write(str(filepath_mha), py2mat(a))
    b, _ = matlab.metaimageio.py.read(str(filepath_mha), nargout=2)
    b = mat2py(b)
    np.testing.assert_almost_equal(b, a)
